package com.link.core.util.redissonUtils;

import com.link.core.basic.service.BasicServiceException;
import com.link.core.util.BeanTools;
import com.link.core.util.RedisUtil;
import com.link.core.util.StringUtils;
import com.link.core.util.dynamicredis.RedisSource;
import com.link.core.util.redisclient.LinkRedisClient;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.concurrent.TimeUnit;
import org.redisson.api.RLock;
import org.redisson.api.RedissonClient;

public class RedissonLockUtil {
   public static void deduction(GetDataCallBack getDataCallBack, Map<String, Object> mapArg, String key, Integer lockLength, BigDecimal subNum) throws Exception {
      checkLockExists(getDataCallBack, mapArg, key, lockLength);
      LinkRedisClient redis = null;
      boolean canExec = false;
      List<RLock> rLocks = new ArrayList();

      try {
         redis = RedisUtil.getJedis(RedisSource.DEV);
         RedissonClient redissonClient = (RedissonClient)BeanTools.getBean(RedissonClient.class);
         int index = 1 + (int)(Math.random() * (double)lockLength);
         BigDecimal compareValue = BigDecimal.ZERO;

         for(int i = 0; i < lockLength; index = index % lockLength + 1) {
            RLock rlock = redissonClient.getLock(key + "_lock_" + index);
            if (!rlock.isLocked() || i >= lockLength - 1) {
               boolean lockFlag = rlock.tryLock(50L, 120L, TimeUnit.SECONDS);
               if (!lockFlag) {
                  throw new Exception("获取锁失败：" + key + "_lock_" + index);
               }

               if ("".equals(redis.get(key + "_num_" + index))) {
                  rlock.unlock();
               } else {
                  BigDecimal curValue = new BigDecimal(redis.get(key + "_num_" + index));
                  compareValue = compareValue.add(curValue);
                  if (curValue.compareTo(BigDecimal.ZERO) == 0) {
                     rlock.unlock();
                  } else {
                     rLocks.add(rlock);
                     if (compareValue.compareTo(subNum) > 0) {
                        canExec = true;
                        break;
                     }
                  }
               }
            }

            ++i;
         }

         if (!canExec) {
            throw new Exception("积分积分池已经不够抵扣当前积分，请及时处理。");
         }

         execDeduction(rLocks, subNum);
      } finally {
         unLock(rLocks);
         RedisUtil.returnResource(redis);
      }

   }

   private static void execDeduction(List<RLock> rLocks, BigDecimal subNum) throws Exception {
      LinkRedisClient redis = null;

      try {
         redis = RedisUtil.getJedis(RedisSource.DEV);
         Iterator var3 = rLocks.iterator();

         while(var3.hasNext()) {
            RLock rLock = (RLock)var3.next();
            if (rLock != null) {
               String name = rLock.getName();
               String dataKey = name.replaceAll("lock", "num");
               String value = redis.get(dataKey);
               if (StringUtils.isNull(value)) {
                  throw new Exception("扣减值的时候，获取得到的值为空,key为：" + dataKey);
               }

               BigDecimal valueBig = new BigDecimal(value);
               if (valueBig.compareTo(subNum) > 0) {
                  valueBig = valueBig.subtract(subNum);
                  redis.set(dataKey, String.valueOf(valueBig));
                  break;
               }

               redis.set(dataKey, "0");
               subNum = subNum.subtract(valueBig);
            }
         }
      } finally {
         RedisUtil.returnResource(redis);
      }

   }

   private static void checkLockExists(GetDataCallBack getDataCallBack, Map<String, Object> mapArg, String key, Integer lockLength) throws Exception {
      LinkRedisClient redis = null;

      try {
         redis = RedisUtil.getJedis(RedisSource.DEV);
         RedissonClient redissonClient = (RedissonClient)BeanTools.getBean(RedissonClient.class);
         boolean existsFlag = redis.exists(key + "_exists_flag");
         boolean existsAgainFlag;
         if (existsFlag) {
            String existsValue = redis.get(key + "_exists_flag");
            if (!"Y".equals(existsValue)) {
               RLock rlock = redissonClient.getLock(key + "_exists_flag_lock");
               existsAgainFlag = rlock.tryLock(100L, 120L, TimeUnit.SECONDS);
               if (!existsAgainFlag) {
                  throw new BasicServiceException("等待另一个线程初始化锁失败，锁为：" + key + "_exists_flag");
               }

               rlock.unlock();
            }

            return;
         }

         RLock rlock = redissonClient.getLock(key + "_exists_flag_lock");
         boolean lockFlag = rlock.tryLock(100L, 120L, TimeUnit.SECONDS);
         if (!lockFlag) {
            return;
         }

         existsAgainFlag = redis.exists(key + "_exists_flag");
         if (!existsAgainFlag) {
            redis.set(key + "_exists_flag", "N");
            initLock(getDataCallBack, mapArg, key, lockLength);
            redis.set(key + "_exists_flag", "Y");
            rlock.unlock();
            return;
         }
      } finally {
         RedisUtil.returnResource(redis);
      }

   }

   private static void initLock(GetDataCallBack getDataCallBack, Map<String, Object> mapArg, String key, Integer lockLength) throws Exception {
      LinkRedisClient redis = null;

      try {
         redis = RedisUtil.getJedis(RedisSource.DEV);
         BigDecimal data = getDataCallBack.execute(mapArg);
         initPointToPool(data, key, lockLength);
      } finally {
         RedisUtil.returnResource(redis);
      }

   }

   public static void resetPointPool(BigDecimal pointValue, String key, Integer lockLength) throws Exception {
      ArrayList<RLock> rLocks = new ArrayList();

      try {
         RedissonClient redissonClient = (RedissonClient)BeanTools.getBean(RedissonClient.class);
         if (redissonClient == null) {
            throw new Exception("RedissonClient为空，请联系管理员！");
         } else {
            for(int i = 1; i <= lockLength; ++i) {
               RLock rlock = redissonClient.getLock(key + "_lock_" + i);
               boolean lockFlag = rlock.tryLock(80L, 120L, TimeUnit.SECONDS);
               if (!lockFlag) {
                  throw new Exception("获取锁失败：" + key + "_lock_" + i);
               }

               rLocks.add(rlock);
            }

            initPointToPool(pointValue, key, lockLength);
         }
      } finally {
         unLock(rLocks);
      }
   }

   private static void initPointToPool(BigDecimal pointValue, String key, Integer lockLength) {
      LinkRedisClient redis = null;

      try {
         redis = RedisUtil.getJedis(RedisSource.DEV);
         BigDecimal length = new BigDecimal(lockLength);
         BigDecimal matchData = pointValue.divideAndRemainder(length)[1];
         BigDecimal zhData = pointValue.subtract(matchData);
         BigDecimal singleData = zhData.divide(length);

         for(int i = 1; i <= lockLength; ++i) {
            if (i == lockLength) {
               BigDecimal lastData = singleData.add(matchData);
               redis.set(key + "_num_" + i, String.valueOf(lastData));
            } else {
               redis.set(key + "_num_" + i, String.valueOf(singleData));
            }
         }
      } finally {
         RedisUtil.returnResource(redis);
      }

   }

   private static void unLock(List<RLock> rLocks) {
      Iterator var1 = rLocks.iterator();

      while(var1.hasNext()) {
         RLock rLock = (RLock)var1.next();
         if (rLock != null && rLock.isLocked() && rLock.isHeldByCurrentThread()) {
            rLock.unlock();
         }
      }

   }
}
